import React, { Component } from "react";
import PropTypes from "prop-types";
import { withStyles } from "@material-ui/core/styles";
import Drawer from "@material-ui/core/Drawer";
import CssBaseline from "@material-ui/core/CssBaseline";
import AppBar from "@material-ui/core/AppBar";
import Toolbar from "@material-ui/core/Toolbar";
import Typography from "@material-ui/core/Typography";
import Divider from "@material-ui/core/Divider";
import Print from "@material-ui/icons/Print";
import Save from "@material-ui/icons/Save";
import ToolTip from "@material-ui/core/Tooltip";
import SlicerComponent from "./SlicerComponent";
import { IconButton } from "@material-ui/core";
import { LoadingOverlay } from "react-overlay-loader";
import * as spreadExcel from '@grapecity/spread-excelio';
import saveAs from 'file-saver';
import "react-overlay-loader/styles.css";

const styles = theme => ({
  root: {
    display: "flex"
  },
  appBar: {
    zIndex: theme.zIndex.drawer + 1,
    width: "100%"
  },
  toolbar: theme.mixins.toolbar,
  content: {
    flexGrow: 1,
    backgroundColor: theme.palette.background.default,
    padding: theme.spacing.unit * 1
  },
  grow: {
    flexGrow: 1
  },
  colorPrimary: {
    backgroundColor: "green"
  }
});

class App extends Component {
  constructor(props) {
    super(props);
    this.handlePrint = this.handlePrint.bind(this);
  }
  state = {
    title: "Accounts Receivable Aging Report with Slicers",
    orgName: "Pro Miles Auto Inc.",
    data: require("./../data/agingReportData.json"),
    spread: null,
    loading: false
  };

  updateState = workBook => {
    this.setState({
      spread: workBook
    });
  };

  handlePrint = spread => {
    try {
      this.state.spread.print(this.state.spread.getActiveSheetIndex());
    } catch (e) {
      alert(e);
    }
  };
  handleExport =spread => {
    try {
      let excelIO = new spreadExcel.IO();
    
        var name= this.state.spread.getActiveSheet().name() + ".xlsx";
        var json = JSON.stringify(this.state.spread.toJSON(
          {
            includeBindingSource: true,
          }
        ));

        excelIO.save(json, (blob) => {
        saveAs(blob, name);
        }, (e) => {
        console.log(e);
        });
      
  
    } catch (e) {
      alert(e);
    }
  };


  render() {
    const { classes } = this.props;

    return (
      <div className={classes.root}>
        <CssBaseline />
        <AppBar
          position="fixed"
          className={classes.appBar}
          classes={{
            colorPrimary: classes.colorPrimary
          }}
          color="primary"
        >
          <Toolbar>
            <Typography variant="title" color="inherit" noWrap>
              {this.state.title}
            </Typography>
            <div className={classes.grow} />
            <IconButton color="inherit" onClick={this.handlePrint}>
            <ToolTip title="Print"><Print /></ToolTip>
            </IconButton>
            <IconButton color="inherit" onClick={this.handleExport} >
            <ToolTip title="Export to Excel"><Save/></ToolTip>
            </IconButton>
          </Toolbar>
        </AppBar>
        <Drawer
          className={classes.drawer}
          variant="permanent"
          classes={{
            paper: classes.drawerPaper
          }}
          anchor="left">
          <div className={classes.toolbar} />
          <Divider />
        </Drawer>
        <main className={classes.content}>
          <div className={classes.toolbar} />
          <LoadingOverlay>
          
            <AgingReport
              data={this.state.data}
              title={this.state.title}
              orgName={this.state.orgName}
              onWorkBookInitialized={this.updateState}
            />
          </LoadingOverlay>
        </main>
      </div>
    );
  }
}

App.propTypes = {
  classes: PropTypes.object.isRequired
};

function AgingReport(props) {

  const data = props.data;
  const title = props.title;
  const orgName = props.orgName;
  const onWorkBookInitialized = props.onWorkBookInitialized;
  const hostStyle = {
    width: "100%",
    height: "700px"
  };
  let jsx = null;
  if (slicerComponent === null) {
    slicerComponent = (
      <SlicerComponent
        data={data}
        title={title}
        organizationName={orgName}
        hostStyle={hostStyle}
        onWorkBookInitialized={onWorkBookInitialized.bind(this)}
      />
    );
  }
  jsx = slicerComponent;
  return jsx;
}


let slicerComponent = null;
export default withStyles(styles)(App);
