import React, { Component } from "react";
import { SpreadSheets, Worksheet } from "@grapecity/spread-sheets-react";
import GC from "@grapecity/spread-sheets";
import "@grapecity/spread-sheets-print";
import "@grapecity/spread-sheets/styles/gc.spread.sheets.excel2016colorful.css";
import { NegativeValueFormatter } from "../shared/NegativeValueFormatter";


class SlicerComponent extends Component {
  constructor(props) {
    super(props);
    this.COLS = 12;
    this.hostStyle = props.hostStyle;

    this.customerRowStyle = new GC.Spread.Sheets.Style();
    this.customerRowStyle.backColor = "lightblue";
    this.customerRowStyle.font = "bold normal 15px normal";
  }
  state = {
    records: this.props.data,
    organizaionName: this.props.organizationName,
    title: "Accounts Receivable",
    dateGeneratedOn: new Date(),
    asOfDate: new Date(),
    dateFormat: "dd/MM/yyyy",
    spread: null
  };

  initSpread = workBook => {
    workBook.suspendPaint();
    this.initDataTableSheet(workBook.getSheetFromName("Aging Report Table"));
    this.initLookupTable(workBook.getSheetFromName("LookupTable"));
    this.initSlicerSheet(workBook.getSheetFromName("Report with Slicer"));
    workBook.resumePaint();
    this.setState({
      spread: workBook
    });
    this.props.onWorkBookInitialized(workBook); 
  };

  initDataTableSheet = sheet => {
    sheet.setDataSource(this.state.records);
    sheet.setColumnWidth(0, 120);
    sheet.setColumnWidth(1, 120);
    sheet.setColumnWidth(2, 120);
    sheet.setColumnWidth(3, 120);
    sheet.setColumnWidth(4, 120);
  };

  //#region Report with Slicers
  initSlicerSheet = sheet => {
    this.addReportHeader(sheet);

    sheet.setRowCount(this.state.records.length + 20);
    sheet.setColumnCount(20);

    let tableRowIndex = 4;
    let table = sheet.tables.addFromDataSource(
      "records",
      tableRowIndex,
      0,
      this.state.records
    );
    sheet.setColumnWidth(0, 180);
    sheet.setColumnWidth(1, 120);
    sheet.setColumnWidth(2, 120);
    sheet.setColumnWidth(3, 120);
    sheet.setColumnWidth(4, 170);
    sheet.setColumnWidth(5, 80);
    sheet.setColumnWidth(6, 120);
    

    sheet.tables.resize(
      table.name(),
      new GC.Spread.Sheets.Range(tableRowIndex, 0, this.state.records.length, 7)
    );

    //#region  Add Tables
    let spreadNS = GC.Spread.Sheets;

    let customerCol = new spreadNS.Tables.TableColumn();
    customerCol.name("Customer");
    customerCol.dataField("Customer");

    let invoiceIDCol = new spreadNS.Tables.TableColumn();
    invoiceIDCol.name("Invoice #");
    invoiceIDCol.dataField("Invoice Number");

    let invoiceDtCol = new spreadNS.Tables.TableColumn();
    invoiceDtCol.name("Invoice Date");
    invoiceDtCol.dataField("Invoice Date");

    let balanceCol = new spreadNS.Tables.TableColumn();
    balanceCol.name("Balance");
    balanceCol.dataField("Amount");
    let dueDateCol = new spreadNS.Tables.TableColumn();
    dueDateCol.name("Due Date");
    dueDateCol.dataField("Due Date");

    let daysLateCol = new spreadNS.Tables.TableColumn();
    daysLateCol.name("DaysLate");

    let cateogoryCol = new spreadNS.Tables.TableColumn();
    cateogoryCol.name("Category");

    table.bindColumns([
      customerCol,
      invoiceIDCol,
      invoiceDtCol,
      dueDateCol,
      balanceCol,
      daysLateCol,
      cateogoryCol
    ]);

    table.setColumnDataFormula(5, "=Today() - [Due Date] ");
    table.highlightLastColumn(false);

    table.setColumnDataFormula(
      6,
      `=VLOOKUP([@DaysLate],LookupTable!$A$1:$B$6,2)`
    );

    let categorySlicer = sheet.slicers.add(
      "categorySlicer",
      table.name(),
      "Category"
    );
    categorySlicer.position(new GC.Spread.Sheets.Point(850, 100));
    categorySlicer.style(GC.Spread.Sheets.Slicers.SlicerStyles.dark1());

    let customerSlicer = sheet.slicers.add(
      "customerSlicer",
      table.name(),
      "Customer"
    );
    customerSlicer.position(new GC.Spread.Sheets.Point(850, 400));
    customerSlicer.style(GC.Spread.Sheets.Slicers.SlicerStyles.dark1());
   
    sheet.setColumnVisible(6, false);
    sheet.getRange(-1, 5, -1, 1).formatter(new NegativeValueFormatter());
    sheet.getRange(-1, 4, -1, 1).formatter("$#,#");
    sheet.getRange(-1, 2, -1, 6, GC.Spread.Sheets.SheetArea.viewport).hAlign(GC.Spread.Sheets.HorizontalAlign.right);
    sheet.getRange(-1, 0, -1, 1, GC.Spread.Sheets.SheetArea.viewport).hAlign(GC.Spread.Sheets.HorizontalAlign.left);
    sheet.getRange(1, 2, 1, 2, GC.Spread.Sheets.SheetArea.viewport).vAlign(GC.Spread.Sheets.HorizontalAlign.center);
    sheet.getRange(4, 1, 1, 5, GC.Spread.Sheets.SheetArea.viewport).hAlign(GC.Spread.Sheets.HorizontalAlign.center);
  


    var cfs = sheet.conditionalFormats;
    var style = new GC.Spread.Sheets.Style();
    style.foreColor = "green";
    cfs.addCellValueRule(
      GC.Spread.Sheets.ConditionalFormatting.ComparisonOperators.lessThan,
      30,
      0,
      style,
      [new GC.Spread.Sheets.Range(-1, 5, -1, 1)]
    );

    style = new GC.Spread.Sheets.Style();
    style.foreColor = "red";
    cfs.addCellValueRule(
      GC.Spread.Sheets.ConditionalFormatting.ComparisonOperators.greaterThan,
      90,
      0,
      style,
      [new GC.Spread.Sheets.Range(-1, 5, -1, 1)]
    );

    var lessThanstyle = new GC.Spread.Sheets.Style();
    lessThanstyle.foreColor = "peru";
    cfs.addCellValueRule(
      GC.Spread.Sheets.ConditionalFormatting.ComparisonOperators.between,
      31,
      90,
      lessThanstyle,
      [new GC.Spread.Sheets.Range(-1, 5, -1, 1)]
    );

    var dataBarRule = new GC.Spread.Sheets.ConditionalFormatting.DataBarRule(GC.Spread.Sheets.ConditionalFormatting.ScaleValueType.Number, 500, GC.Spread.Sheets.ConditionalFormatting.ScaleValueType.Number, 12000, "#6891c8", [new GC.Spread.Sheets.Range(-1,4,-1,1)]);
    dataBarRule.showBorder(true);
    dataBarRule.borderColor("#6891c8");
    dataBarRule.dataBarDirection(GC.Spread.Sheets.ConditionalFormatting.BarDirection.LeftToRight);
    dataBarRule.axisPosition(GC.Spread.Sheets.ConditionalFormatting.DataBarAxisPosition.Automatic);
    cfs.addRule(dataBarRule);

  };

  initLookupTable = sheet => {
    let lookUpRowIndex = -1,
      lookUpColIndex = 0;
    let row = lookUpRowIndex;
    sheet.getCell(++row, lookUpColIndex).text("Lookup");
    sheet.getCell(row, lookUpColIndex + 1).text("Category");
    sheet.getCell(++row, lookUpColIndex).formula("-90");
    sheet.getCell(row, lookUpColIndex + 1).text("Current");
    sheet.getCell(++row, lookUpColIndex).text("1");
    sheet.getCell(row, lookUpColIndex + 1).text("[1-30]");
    sheet.getCell(++row, lookUpColIndex).text("31");
    sheet.getCell(row, lookUpColIndex + 1).text("[31-60]");
    sheet.getCell(++row, lookUpColIndex).text("61");
    sheet.getCell(row, lookUpColIndex + 1).text("[61-90]");
    sheet.getCell(++row, lookUpColIndex).text("91");
    sheet.getCell(row, lookUpColIndex + 1).text("91+");
  };
  //#endregion
  addReportHeader = sheet => {
    sheet.setColumnCount(this.COLS);
    sheet.setRowCount(10000);

    sheet
      .getCell(1, 0)
      .text(this.state.title)
      .font("bold normal 20px normal")
      .vAlign(GC.Spread.Sheets.VerticalAlign.center);
    sheet.addSpan(0, 0, 1, 2);
    sheet
      .getCell(0, 0)
      .text(this.state.organizaionName)
      .font("bold normal 20px normal")
      .vAlign(GC.Spread.Sheets.VerticalAlign.center);
    sheet.addSpan(1, 0, 1, 2);
    sheet
      .getCell(1, 2)
      .text("As of:")
      .font("bold normal 15px normal");
    sheet
      .getCell(1, 3)
      .text(this.state.dateGeneratedOn)
      .formatter(this.state.dateFormat)
      .font("bold normal 15px normal");

    sheet.setRowHeight(0, 40);
    sheet.setRowHeight(1, 28);
    sheet.setColumnWidth(0, 150);
    sheet.setColumnWidth(1, 120);
    sheet.setColumnWidth(2, 120);
    sheet.setColumnWidth(3, 120);
    sheet.setColumnWidth(4, 80);
    sheet.setColumnWidth(5, 80);
    sheet.setColumnWidth(6, 80);
    sheet.setColumnWidth(7, 80);
    sheet.setColumnWidth(8, 80);
    sheet.setColumnWidth(9, 80);
    sheet.setColumnWidth(10, 80);
    sheet.setColumnWidth(11, 80);

    let defaultStyle = new GC.Spread.Sheets.Style();
    defaultStyle.foreColor = "Black";
    defaultStyle.hAlign = GC.Spread.Sheets.HorizontalAlign.center;
    sheet.setDefaultStyle(defaultStyle);
  };
  render() {
    return (
      <div>
        <SpreadSheets
          hostStyle={this.hostStyle}
          workbookInitialized={this.initSpread.bind(this)}
        >
          <Worksheet name="Report with Slicer" />
          <Worksheet name="LookupTable" />
          <Worksheet name="Aging Report Table" />
        </SpreadSheets>
      </div>
    );
  }
}

export default SlicerComponent;
